function varargout = SAOptions(varargin)
%SAOPTIONS Control code for the Spatial Analysis Options Window
%  SAOptions by itself will create a new SAOptions window with the default
%  values loaded into each uicontrol.  SAOptions is always created as a
%  modal window, so it must be closed in order to continue.
%
%  SAOptions(options)  Creates a new SAOptions window and loads the values
%  specified in options into the appropriate uicontrol.  Options is a
%  structure with field names equal to the Tag name of the uicontrol.
%
% See also: SPATIALANALYSIS

%  Below is the help information generated by the guide editor in Matlab
%
%      SAOPTIONS, by itself, creates a new SAOPTIONS or raises the existing
%      singleton*.
%
%      H = SAOPTIONS returns the handle to a new SAOPTIONS or the handle to
%      the existing singleton*.
%
%      SAOPTIONS(OPTIONS) Creates a new SAOPTIONS and loads the values in
%      the structure OPTIONS into the appropriate boxes.
% 
%      SAOPTIONS('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in SAOPTIONS.M with the given input arguments.
%
%      SAOPTIONS('Property','Value',...) creates a new SAOPTIONS or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before SAOptions_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to SAOptions_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Copyright 2002-2003 The MathWorks, Inc.

% Edit the above text to modify the response to help SAOptions

% Last Modified by GUIDE v2.5 25-Mar-2002 12:19:15

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @SAOptions_OpeningFcn, ...
    'gui_OutputFcn',  @SAOptions_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    if(strcmp(varargin{1},'tabcallbk'))
        tabcallbk(varargin{2:6})
        return
    else
        gui_State.gui_Callback = str2func(varargin{1});
    end
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before SAOptions is made visible.
function SAOptions_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to SAOptions (see VARARGIN)

% Choose default command line output for SAOptions
tabnames = {'General','NN/Delaunay','Voronoi','Autocorrelation',...
    'F,G,K','Random Data'};

tabDims{1} = [65 79 70 88 65 80];%[75*ones(size(tabnames));
tabDims{2} = [20];
callback = 'SAOptions';
sheetDims = [465 350]; %[465 315];
offsets = [5 5 5 75];
pos1 = get(handles.figure1,'position');
h0 = tabdlg('create',tabnames,tabDims,callback,sheetDims,offsets,1,{},handles.figure1);
pos2 = get(h0,'position');
% Set the position to be the middle of the screen
screensize = get(0,'ScreenSize');
pos2(1) = max((screensize(3)-pos2(3))/2,0);
pos2(2) = max((screensize(4)-pos2(4))/2,0);
set(h0,'Tag',mfilename,'Name','3D Spatial Analysis Options',...
    'WindowStyle','modal','position',pos2)
set(h0,'visible','on')
setappdata(h0,'CurrentTab',1)

% Form the boxes and text in each section of the tabbed figure


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  FIRST TAB OPTIONS                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Define a position matrix
posx = [30 245 340];
posy = [9:-1:1]*35+70;
dx = [190 85 50];
dy = 20;

uicontrolstrings = {...
    'Sample space max x','1000','';...
    'Sample space max y','1000','';...
    'Sample space max z','1000','';...
    'Marker size','13','';...
    'Decimal Precision','4','';...
    'Plot Transparency','0.5',''};
uitooltipstrings = {...
    'Sample space boundary in the x-direction';...
    'Sample space boundary in the y-direction';...
    'Sample space boundary in the z-direction';...
    'Marker size for the plots';...
    'Decimal precision for the data table';...
    'Voronoi diagram plot transparency (0-1)'};
uitags = {...
    'MaxX';...
    'MaxY';...
    'MaxZ';...
    'MarkerSize';...
    'Precision';...
    'Transparency'};

[A,B] = size(uicontrolstrings);
h1 = ones(B,A);

for y=1:A
    for x=1:B
        switch(x)
            case 1
                h1(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','tagTXT', ...
                    'TooltipString',uitooltipstrings{y}, ...
                    'visible','on');
            case 2
                h1(x,y) = uicontrol('Parent',h0, ...
                    'BackgroundColor',[1 1 1], ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','edit', ...
                    'Tag',uitags{y}, ...
                    'TooltipString',uitooltipstrings{y},...
                    'UserData',uicontrolstrings{y,x}, ...
                    'visible','on');
            case 3
                h1(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','units', ...
                    'visible','on');
        end
    end
end

% Add the choice for units.  Since the default font for a pc is MS San
% Serif, we can use the mu symbol for microns.  This font is not the
% default for non pc machines and may not even be availabale.  So, check to
% see if we are on a pc and if not, spell out microns
y = y+1;
% AvailFonts = listfonts;
% if(any(strcmpi(AvailFonts,'MS Sans Serif')))
%     Units = {'A';'nm';'m';'cm';'m'};
% elseif(any(strcmpi(AvailFonts,'Arial')))
%     Units = {'A';'nm';'m';'cm';'m'};
% end

if(ispc)
    Units = {'A';'nm';'m';'cm';'m'};
else
    Units = {'A';'nm';'um';'cm';'m'};
end

h1(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Units', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose the units', ...
    'visible','on');
h1(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',Units, ...
    'Style','popupmenu', ...
    'Tag','Units', ...
    'TooltipString','Choose the units', ...
    'UserData',3, ...
    'Value',3, ...
    'visible','on');

y = y+1;
h1(1:3,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) sum(dx) dy], ...
    'String','Show Tips on Startup', ...
    'Style','Checkbox', ...
    'Tag','TipsCB', ...
    'TooltipString','Check the box to show the tip dialog at startup', ...
    'value',1, ...
    'visible','on');

% Set the checkbox's value (check state)
if(ispref('SpatialAnalysisTips','Display'))
    set(h1(1,y),'value',getpref('SpatialAnalysisTips','Display'),'UserData',1)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SECOND TAB OPTIONS                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uicontrolstrings = {...
    'Nearest neighbor bins','15','';...
    'Nearest neighbor bin size','20','';...
    'Delaunay segment len bins','15','';...
    'Delaunay segment len bin size','20','';...
    'Delaunay facet area bins','15','';...
    'Delaunay facet area bin size','3e4','';...
    'Delaunay tetrahedron volume bins','15','';...
    'Delaunay tetrahedron volume bin size','1e6',''};
uitooltipstrings = {...
    'Number of nearest neighbor bins in the histogram';...
    'Nearest neighbor bin size';...
    'Number of Delaunay segment bins in the histogram';...
    'Delaunay segment bin size';...
    'Number of Delaunay facet area bins in the histogram';...
    'Delaunay facet area bin size';...
    'Number of Delaunay tetrahedron volume bins in the histogram';...
    'Delaunay tetrahedron volume bin size'};
uitags = {...
    'NumNNBins';...
    'NNBinSize';...
    'NumDelaunaySegLenBins';...
    'DelaunaySegLenBinSize';...
    'NumDelaunayAreaBins';...
    'DelaunayAreaBinSize';...
    'NumDelaunayVolBins';...
    'DelaunayVolBinSize'};

[A,B] = size(uicontrolstrings);
h2 = ones(B,A);
for y=1:A
    for x=1:B
        switch(x)
            case 1
                h2(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','tagTXT', ...
                    'TooltipString',uitooltipstrings{y}, ...
                    'visible','off');
            case 2
                h2(x,y) = uicontrol('Parent',h0, ...
                    'BackgroundColor',[1 1 1], ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','edit', ...
                    'Tag',uitags{y}, ...
                    'TooltipString',uitooltipstrings{y},...
                    'UserData',uicontrolstrings{y,x}, ...
                    'visible','off');
            case 3
                h2(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','units', ...
                    'visible','off');
        end
    end
end

% Add the choice for point filtering
y = y+1;
NNFilter = {'None';'Convex Hull';'Infected Points';'Double Infected Points'};
h2(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Filter', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose a filter for the nearest neighbor/Delaunay calculation', ...
    'visible','off');
h2(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',NNFilter, ...
    'Style','popupmenu', ...
    'Tag','NNFilter', ...
    'TooltipString','Choose a filter for the nearest neighbor/Delaunay calculation', ...
    'UserData',3, ...
    'value',3, ...
    'visible','off');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  THIRD TAB OPTIONS                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uicontrolstrings = {...
    'Voronoi volume bins','15','';...
    'Voronoi volume bin size','2e5','';...
%    'Voronoi volume max freq.','30','%';...
    'Voronoi area bins','15','';...
    'Voronoi area bin size','1e4',''};
%    'Voronoi area max freq.','30','%'};
uitooltipstrings = {...
    'Number of Voronoi volume bins';...
    'Voronoi volume bin size';...
%    'Maximum frequency bin for the voronoi volumes';...
    'Number of Voronoi area bins';...
    'Voronoi area bin size'};
%    'Maximum frequency bin for the voronoi area'};
uitags = {...
    'NumVoronoiVolBins';...
    'VoronoiVolBinSize';...
%    'VoronoiVolMaxFreq';...
    'NumVoronoiAreaBins';...
    'VoronoiAreaBinSize'};
%    'VoronoiAreaMaxFreq'};

[A,B] = size(uicontrolstrings);
h3 = ones(B,A);
for y=1:A
    for x=1:B
        switch(x)
            case 1
                h3(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','tagTXT', ...
                    'TooltipString',uitooltipstrings{y}, ...
                    'visible','off');
            case 2
                h3(x,y) = uicontrol('Parent',h0, ...
                    'BackgroundColor',[1 1 1], ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','edit', ...
                    'Tag',uitags{y}, ...
                    'TooltipString',uitooltipstrings{y},...
                    'UserData',uicontrolstrings{y,x}, ...
                    'visible','off');
            case 3
                h3(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','units', ...
                    'visible','off');
        end
    end
end

% Add the choice for point filtering
y = y+1;
VoronoiFilter = {'Convex Hull';'Infected Points';'Double Infected Points'};
h3(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Filter', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose a filter for the voronoi calculation', ...
    'visible','off');
h3(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',VoronoiFilter, ...
    'Style','popupmenu', ...
    'Tag','VoronoiFilter', ...
    'TooltipString','Choose a filter for the voronoi calculation', ...
    'UserData',2, ...
    'value',2,...
    'visible','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  FOURTH TAB OPTIONS                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uicontrolstrings = {...
    'Autocorrelation radial bins','20','';...
    'Autocorrelation max radius','200','';...
    'Tri-histogram theta bins','36','';...
    'Tri-histogram phi bins','18','';...
    'Marker Size','10',''};
uitooltipstrings = {...
    'Number of radial autocorrelation bins';...
    'Maximum radii in the autocorrelation';...
    'Number of theta bins in the tri-histogram';...
    'Number of phi bins in the tri-histogram';...
    'Marker Size for the autocorrelation plot'};
uitags = {...
    'NumACBins';...
    'MaxRadiusAC';...
    'ACThetaBins';...
    'ACPhiBins';...
    'MarkerSizeAC'};

[A,B] = size(uicontrolstrings);
h4 = ones(B,A+1);
for y=1:A
    for x=1:B
        switch(x)
            case 1
                h4(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','tagTXT', ...
                    'TooltipString',uitooltipstrings{y}, ...
                    'visible','off');
            case 2
                h4(x,y) = uicontrol('Parent',h0, ...
                    'BackgroundColor',[1 1 1], ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','edit', ...
                    'Tag',uitags{y}, ...
                    'TooltipString',uitooltipstrings{y},...
                    'UserData',uicontrolstrings{y,x}, ...
                    'visible','off');
            case 3
                h4(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','units', ...
                    'visible','off');
        end
    end
end

% Add the choice for edge corrections
y = y+1;
Filter = {'Convex Hull';'Max radius';'None'};
h4(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Filter', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose an filtering method for the points', ...
    'visible','off');
h4(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',Filter, ...
    'Style','popupmenu', ...
    'Tag','FilterAC', ...
    'TooltipString','Choose an filtering method for the points', ...
    'UserData',3, ...
    'value',3, ...
    'visible','off');

% Add the choice for edge corrections
y = y+1;
EdgeCorrections = {'Fractional';'None'};
h4(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Edge correction', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose an edge correction algorithm', ...
    'visible','off');
h4(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',EdgeCorrections, ...
    'Style','popupmenu', ...
    'Tag','EdgeCorrectionAC', ...
    'TooltipString','Choose an edge correction algorithm', ...
    'UserData',1, ...
    'value',1, ...
    'visible','off');

% % Add a checkbox that says whether or not we should plot the AC
% y = y+1;
% h4(:,y) = uicontrol('Parent',h0, ...
%     'HorizontalAlignment','left', ...
%     'ListboxTop',0, ...
%     'Position',[posx(1) posy(y) sum(dx) dy], ...
%     'String','Plot autocorrelation', ...
%     'Style','checkbox', ...
%     'Tag','PLOTAC', ...
%     'TooltipString','Plot the autocorrelation points', ...
%     'visible','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  FIFTH TAB OPTIONS                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Build the 3 panels to hold information for F,G,K
h5(1) = uipanel('Parent',h0, ...
    'BackgroundColor',get(h0,'color'),...
    'Position',[.03 .65 .94 .25], ...
    'Title','F-Function', ...
    'Tag','FFcnPanel', ...
    'visible','off');
h5(2) = uipanel('Parent',h0, ...
    'BackgroundColor',get(h0,'color'),...
    'Position',[.03 .425 .94 .20], ...
    'Title','G-Function', ...
    'Tag','FFcnPanel', ...
    'visible','off');
h5(3) = uipanel('Parent',h0, ...
    'BackgroundColor',get(h0,'color'),...
    'Position',[.03 .20 .94 .20], ...
    'Title','K-Function', ...
    'Tag','FFcnPanel', ...
    'visible','off');

uicontrolstrings = {...
    'Range','200';...
    'Number of points','30';...
    'Number of F-divisions','100'};
uitooltipstrings = {...
    'Range over which to calculate the F-Function';...
    'Number of points to use in the given range';...
    'Number of voxels, or divisions of the sample window'};
uitags = {...
    'FFcnRange';...
    'FFcnNumPoints';...
    'fdivisions'};

% Setup a normalized grid pattern
posx2 = [.05 .50 .75];
posy2 = [.75 .45 .1];
dx2 = [.35 .25];
dy2 = .23;

for y = 1:length(uitags)
    h = uicontrol('Parent',h5(1), ...
        'HorizontalAlignment','left', ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(1) posy2(y)-.05 dx2(1) dy2], ...
        'String',uicontrolstrings{y,1}, ...
        'Style','text', ...
        'Tag','tagTXT', ...
        'TooltipString',uitooltipstrings{y}, ...
        'visible','on');

    h = uicontrol('Parent',h5(1), ...
        'BackgroundColor',[1 1 1], ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(2) posy2(y) dx2(2) dy2], ...
        'String',uicontrolstrings{y,2}, ...
        'Style','edit', ...
        'Tag',uitags{y}, ...
        'TooltipString',uitooltipstrings{y},...
        'UserData',uicontrolstrings{y,2}, ...
        'visible','on');
end

uicontrolstrings = {...
    'Range','200';...
    'Number of points','30'};
uitooltipstrings = {...
    'Range over which to calculate the G-Function';...
    'Number of points to use in the given range'};
uitags = {...
    'GFcnRange';...
    'GFcnNumPoints'};

% Setup a normalized grid pattern
posy2 = [.60 .1];
dx2 = [.35 .25];
dy2 = .30;

for y = 1:length(uitags)
    h = uicontrol('Parent',h5(2), ...
        'HorizontalAlignment','left', ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(1) posy2(y)-.05 dx2(1) dy2], ...
        'String',uicontrolstrings{y,1}, ...
        'Style','text', ...
        'Tag','tagTXT', ...
        'TooltipString',uitooltipstrings{y}, ...
        'visible','on');

    h = uicontrol('Parent',h5(2), ...
        'BackgroundColor',[1 1 1], ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(2) posy2(y) dx2(2) dy2], ...
        'String',uicontrolstrings{y,2}, ...
        'Style','edit', ...
        'Tag',uitags{y}, ...
        'TooltipString',uitooltipstrings{y},...
        'UserData',uicontrolstrings{y,2}, ...
        'visible','on');
end

uicontrolstrings = {...
    'Range','200';...
    'Number of points','30'};
uitooltipstrings = {...
    'Range over which to calculate the K-Function';...
    'Number of points to use in the given range'};
uitags = {...
    'KFcnRange';...
    'KFcnNumPoints'};

for y = 1:length(uitags)
    h = uicontrol('Parent',h5(3), ...
        'HorizontalAlignment','left', ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(1) posy2(y)-.05 dx2(1) dy2], ...
        'String',uicontrolstrings{y,1}, ...
        'Style','text', ...
        'Tag','tagTXT', ...
        'TooltipString',uitooltipstrings{y}, ...
        'visible','on');

    h = uicontrol('Parent',h5(3), ...
        'BackgroundColor',[1 1 1], ...
        'ListboxTop',0, ...
        'Units','normalized',...
        'Position',[posx2(2) posy2(y) dx2(2) dy2], ...
        'String',uicontrolstrings{y,2}, ...
        'Style','edit', ...
        'Tag',uitags{y}, ...
        'TooltipString',uitooltipstrings{y},...
        'UserData',uicontrolstrings{y,2}, ...
        'visible','on');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SIXTH TAB OPTIONS                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uicontrolstrings = {...
    '','','';...
    'Number of Random Points','1000','';...
    'Mean minimal distance','22','';...
    'Standard deviation','2','';...
    'Smallest permissible distance','2',''};
uitooltipstrings = {...
    '';...
    'Number of points in the simulated data';...
    'Average minimum distance between random points';...
    'Standard deviation of the distance between random points';...
    'Minimum acceptable distance between random points'};
uitags = {...
    '';...
    'NumRandPoints';...
    'RandomAvgDist';...
    'RandomSTD';...
    'RandomMinDist'};

[A,B] = size(uicontrolstrings);
h6 = ones(B,A);

% Add the choice for simulated data type
y = 1;
SimulatedType = {'Random';'HCP'};
h6(1,y) = uicontrol('Parent',h0, ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(1) posy(y) dx(1) dy], ...
    'String','Simulated Data Type', ...
    'Style','text', ...
    'Tag','tagTXT', ...
    'TooltipString','Choose a type of simulated data', ...
    'visible','off');
h6(2:3,y) = uicontrol('Parent',h0, ...
    'BackgroundColor',[1 1 1],...
    'Callback','SAOptions(''SimulatedDataType_Callback'',gcbo,[],guidata(gcbf))', ...
    'HorizontalAlignment','left', ...
    'ListboxTop',0, ...
    'Position',[posx(2) posy(y) sum(dx(2:3)) dy], ...
    'String',SimulatedType, ...
    'Style','popupmenu', ...
    'Tag','SimulatedDataType', ...
    'TooltipString','Random or Hexagonal Close Packed', ...
    'UserData',1, ...
    'Value',1, ...
    'visible','off');

for y=2:A
    for x=1:B
        switch(x)
            case 1
                h6(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','tagTXT', ...
                    'TooltipString',uitooltipstrings{y}, ...
                    'visible','off');
            case 2
                h6(x,y) = uicontrol('Parent',h0, ...
                    'BackgroundColor',[1 1 1], ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','edit', ...
                    'Tag',uitags{y}, ...
                    'TooltipString',uitooltipstrings{y},...
                    'UserData',uicontrolstrings{y,x}, ...
                    'visible','off');
            case 3
                h6(x,y) = uicontrol('Parent',h0, ...
                    'HorizontalAlignment','left', ...
                    'ListboxTop',0, ...
                    'Position',[posx(x) posy(y) dx(x) dy], ...
                    'String',uicontrolstrings{y,x}, ...
                    'Style','text', ...
                    'Tag','units', ...
                    'visible','off');
        end
    end
end

% DONE BUTTON
hB = uicontrol('Parent',h0, ...
    'Callback',[mfilename,'(''DONE'',gcbo,[],guihandles)'], ...
    'ListboxTop',0, ...
    'Position',[30 25 100 30], ...
    'String','DONE', ...
    'Tag','DoneButton');

% RESET BUTTON
hB = uicontrol('Parent',h0, ...
    'Callback',[mfilename,'(''RESET'',gcbo,[],guihandles)'], ...
    'ListboxTop',0, ...
    'Position',[185 25 100 30], ...
    'String','RESET', ...
    'Tag','ResetButton');


% CANCEL BUTTON
hB = uicontrol('Parent',h0, ...
    'Callback',[mfilename,'(''CANCEL'',gcbo,[],guihandles)'], ...
    'ListboxTop',0, ...
    'Position',[340 25 100 30], ...
    'String','CANCEL', ...
    'Tag','CancelButton');

uicontrolhandles(1) = {h1};
uicontrolhandles(2) = {h2};
uicontrolhandles(3) = {h3};
uicontrolhandles(4) = {h4};
uicontrolhandles(5) = {h5};
uicontrolhandles(6) = {h6};
setappdata(gcf,'uicontrolhandles',uicontrolhandles)


% Update handles structure
handles = guihandles;
handles.output = hObject;
handles.cbf = gcbf;
guidata(hObject, handles);

% Get the options if they were passed
if(length(varargin))
    options = varargin{1};
    LoadOptions(hObject, eventdata, handles, options)
end

%--------------------------------------------------------------------------
function LoadOptions(hObject, eventdata, handles, varargin)
% This action will load the options
options = varargin{1};

% field1 represents the fields in the options that were passed to the
% function and field2 represents all the fields in the options window
fields1 = fieldnames(options);
fields2 = fieldnames(handles);

for i = 1:length(fields1)
    if(any(strcmp(fields1{i},fields2)))
        hO = findobj('tag',fields1{i});
        switch(get(hO,'style'))
            case 'edit'
                set(hO,'string',getfield(options,fields1{i}));
            case 'checkbox'
                set(hO,'value',getfield(options,fields1{i}));
            case 'popupmenu'
                choices = get(hO,'string');
                choice = find(strcmp(choices,getfield(options,fields1{i})));
                if(isempty(choice))
                    choice = 1;
                end
                set(hO,'value',choice)
            otherwise
                
        end
    end
end

set(gcf,'UserData',options)

% Check the value of SimulatedDataType
choices = get(handles.SimulatedDataType,'string');
SimulatedDataType = choices{get(handles.SimulatedDataType,'value')};
if(strcmp(SimulatedDataType,'HCP'))
    set(handles.NumRandPoints,'enable','off')
end

% --- Outputs from this function are returned to the command line.
function varargout = SAOptions_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

%--------------------------------------------------------------------------
function tabcallbk(pressedTab,pressedTabNum,previousTab,previousTabNum,hfig)
% If this option was chosen, then we need to hide the respective boxes and text
uicontrolhandles = getappdata(gcf,'uicontrolhandles');

set(uicontrolhandles{previousTabNum},'visible','off')
set(uicontrolhandles{pressedTabNum},'visible','on')
setappdata(gcbf,'CurrentTab',pressedTabNum)

%--------------------------------------------------------------------------
function CANCEL(hObject,eventdata,handles)
% If the user selected the cancel button then we need to return control
% back to the calling figure without updating the options.
close(gcbf)

%--------------------------------------------------------------------------
function RESET(hObject,eventdata,handles)
% If the user selected the reset button then we need to load the default
% data into the window
CurTab = getappdata(gcbf,'CurrentTab');

% Get all the control handles from the figure window
uicontrolhandles = getappdata(gcf,'uicontrolhandles');

% Get all the handles for the second column of handles (data column)
PanelHandles = uicontrolhandles{CurTab};
NumRows = size(PanelHandles,1);
if(NumRows>1)
    TabHandles = uicontrolhandles{CurTab}(2,:);
else
    TabHandles = cell2mat(get(PanelHandles,'Children'));
end

for i = 1:length(TabHandles)
    switch(get(TabHandles(i),'style'))
        case 'edit'
            set(TabHandles(i),'string',get(TabHandles(i),'UserData'));
        case {'checkbox','popupmenu'}
            set(TabHandles(i),'value',get(TabHandles(i),'UserData'));
        otherwise
            % We don't have a valid uicontrol type
    end
end



%--------------------------------------------------------------------------
function DONE(hObject,eventdata,handles)
% If the user selected the done button, then we need to get all the values
% from the boxes and return function to the callling figure
options = get(gcf,'UserData');

% Fields are the fields in the figure window
fields = fieldnames(handles);

% Skip the first field, that is for the figure itself
for i = 2:length(fields)
    % Get the type of field we are working with
    fieldtype = get(getfield(handles,fields{i}),'type');
    
    % If the fieldtype is a cell array, or the field is not of type 
    % uicontrol, then we are dealing with more than one field and we should
    % continue to the next field
    if(iscell(fieldtype))
        continue
    elseif(~strcmp(fieldtype,'uicontrol'))
        continue
    end
    
    % Get the field syle.  If there are multiple fields of the same name,
    % then fieldstyle will be a cell array and we should just skip it
    fieldstyle = get(getfield(handles,fields{i}),'style');
    if(iscell(fieldstyle))
        continue
    end
    % Get the value in the field depending on it's style
    switch(fieldstyle)
        case 'edit'
            % If there is a value in the string field, then save it in the
            % options structure, if not, then we leave it as it was
            if(~isempty(str2num(get(getfield(handles,fields{i}),'string'))))
                options = setfield(options,fields{i},...
                    str2num(get(getfield(handles,fields{i}),'string')));
            end
        case 'checkbox'
            % If we have a checkbox, then set the value (flag)
            options = setfield(options,fields{i},...
                get(getfield(handles,fields{i}),'value'));
        case 'popupmenu'
            % If we  have a popupmenu then get the choices and the value
            choices = get(getfield(handles,fields{i}),'string');
            choice = choices{get(getfield(handles,fields{i}),'value')};
            options = setfield(options,fields{i},choice);

        case 'otherwise'
            % Nothing yet
    end
end

% Set the tips checkbox separately
setpref('SpatialAnalysisTips','Display',get(handles.TipsCB,'value'))

%Do some data checking
options.Transparency = min(max(options.Transparency,0),1);

% Check to see what the original calling figure is.  If the orginal
% callback figure was THIS figure, then close the figure
% Get the handles from the GUI
handles = guidata(gcf);
if(isempty(handles.cbf) || handles.cbf==gcf)
    close(gcf)
else
    % If we called SAOptions from another figure...
    % Update the userdata and Make the figure invisible so that the calling
    % function cal get the data
    set(gcf,'UserData',options,'visible','off')
end

%--------------------------------------------------------------------------
function SimulatedDataType_Callback(hObject,eventdata,handles)
% This function will enable or disable the number of random points box
% based on the type of simulated data we are using
switch(get(hObject,'value'))
    case 1 % Uniformly random
        set(handles.NumRandPoints,'enable','on')
    case 2 % Hexagonal Close Packed
        set(handles.NumRandPoints,'enable','off')
end

    

