function [cmp] = cmpstruct(struct1,struct2)
%CMPSTRUCT  Compares two structures to see if they are identical
%  CMPSTRUCT(STRUCT1,STRUCT2)  Compares each of the fields in struct1 to
%  the fields in struct2 to see if the two structures are identical.  If
%  each field is not the same or the structures do not have the same fields
%  the function will return a 0. Otherwise, if they are identical a 1 is
%  returned
%
%   See also STRCMP, FIELDNAMES, GETFIELD

% If they are not even structures then we should not proceed
if(~isstruct(struct1) | ~isstruct(struct2))
    cmp = 0;
    return
end

fieldnames1 = fieldnames(struct1);
fieldnames2 = fieldnames(struct2);


% If the number of fields are not the same then we know they are not equal
if(length(fieldnames1)~=length(fieldnames2))
    cmp = 0;
    return
end

% Go through each fieldname  First see if the field names are the same,
% then, if they are, get the value for each field and compare them.  Need
% to compare the values depending on the type of data
for i = 1:length(fieldnames1)
    scmp = strcmp(fieldnames1{i},fieldnames2);
    if(~any(scmp))
        cmp = 0;
        return
    else
        A = find(scmp);
        a = getfield(struct1,fieldnames1{i});
        b = getfield(struct2,fieldnames2{A});
        if(isstruct(a))
            cmp = cmpstruct(a,b);
        elseif(isstr(a))
            cmp = strcmp(a,b);
        else
            cmp = (a==b);
        end
        if(cmp==0)
            return
        end
    end
end

% If we made it through all that then the structures are the same
cmp = 1;
