function varargout = deallayers(layers,varargin)
%DEALLAYERS  Deal layer structure field contents
%  [A,B,C,...] = DEAL(LAYERS,'FIELD1','FIELD2',...)  Returns the contents 
%  of the fields from structure layers. For example:
%
%     A = layers(:).name
%
%  If any of the fields are empty [], the function simply skips over those
%  fields and proceeds to the next element in the array
%
%  layers must be an array of structures with fields 'field1','field2',etc

% Check to make sure the user has the correct number of output arguments
if(nargout~=length(varargin))
    error('Number of output arguments must equal number of requested fields')
end

% Make sure we passed an array of structures
[A,B] = size(layers);
if(A~=1 & B~=1)
    error('layers must be an array of structures, not a matrix')
end

% Go through each field requested by the user
for i = 1:length(varargin)
    % make sure the requested field name is valid
    if(~isfield(layers,varargin(i)))
        error([varargin{1},' is not a valid field name'])
    end
    field = [];
    for j = 1:length(layers)
        f = getfield(layers(j),varargin{i});
        if(~isempty(f))
            [A,B] = size(f);
            if(B~=1 & ~ischar(f))
                f = f';
            end
            if(ischar(f))
                field = [field;{f}];
            else
                field = [field;f];
            end
        end
    end
    varargout(i) = {field};
end
